package w5.ex1.frequencytable1.sol;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Random;
import java.util.Set;

/*
 * Timing test of the two versions, one using IncrementableInteger (of sorts)
 * and the other using Integer.
 */
public class HistogramTest {
	public static void main(String[] args) {
		final int nOfInts = 10000000;
		Integer[] ints = new Integer[nOfInts];
		Random r = new Random(1);
		Set<Integer> intSet = new HashSet<Integer>();

		for (int i = 0; i < nOfInts; i++) {
			ints[i] = Integer.valueOf(r.nextInt(nOfInts / 10));
			intSet.add(ints[i]);
		}
		int unique = intSet.size();
		System.out.println("Out of " + nOfInts + ", " + unique
				+ " were unique.");
		intSet = null;
		System.gc();

		@SuppressWarnings("unchecked")
		Histogram<Integer>[] histograms =
				new Histogram[] {
						new Histogram1<Integer>(),
						new Histogram2<Integer>()
		};
		for (int i = 0; i < 10; i++) {
			for (Histogram<Integer> h : histograms) {
				long start = System.nanoTime();
				for (Integer s : ints) {
					h.increment(s);
				}
				long end = System.nanoTime();
				System.out.println(h.getClass() + ": Insertion of "
						+ nOfInts
						+ " ints, out of which " +
						unique + " were unique took " + (end - start)
						/ 1000000000.0 + " seconds");
				h.clear();
				System.gc();
			}
		}
	}

	private interface Histogram<T> {
		public void increment(T k);

		public void add(T k, long v);

		public long get(T k);

		public void clear();
	}

	private static class Histogram1<T> implements Histogram<T> {
		private Map<T, IncInt> hist;

		public Histogram1() {
			this.hist = new HashMap<T, IncInt>();
		}

		private class IncInt {
			private long val;

			public IncInt(long initValue) {
				this.val = initValue;
			}

			public long getValue() {
				return this.val;
			}

			public long inc() {
				return ++this.val;
			}

			public long add(long v) {
				return this.val += v;
			}
		}

		@Override
		public void increment(T k) {
			IncInt e = this.hist.get(k);
			if (e == null) {
				this.hist.put(k, new IncInt(1));
			} else {
				e.inc();
			}
		}

		@Override
		public void add(T k, long v) {
			IncInt e = this.hist.get(k);
			if (e == null) {
				this.hist.put(k, new IncInt(v));
			} else {
				e.add(v);
			}
		}

		@Override
		public long get(T k) {
			IncInt e = this.hist.get(k);
			if (e != null) {
				return e.getValue();
			}
			return 0;
		}

		@Override
		public void clear() {
			this.hist.clear();
		}
	}

	private static class Histogram2<T> implements Histogram<T> {
		private Map<T, Long> hist;

		public Histogram2() {
			this.hist = new HashMap<T, Long>();
		}

		@Override
		public void increment(T k) {
			Long e = this.hist.get(k);
			//this.hist.get(null);
			if (e == null) {
				this.hist.put(k, Long.valueOf(1L));
			} else {
				this.hist.put(k, Long.valueOf(e.longValue() + 1L));
			}
		}

		@Override
		public void add(T k, long v) {
			Long e = this.hist.get(k);
			if (e == null) {
				this.hist.put(k, Long.valueOf(v));
			} else {
				this.hist.put(k, Long.valueOf(e.longValue() + v));
			}
		}

		@Override
		public long get(T k) {
			Long e = this.hist.get(k);
			if (e != null) {
				return e.longValue();
			}
			return 0;
		}

		@Override
		public void clear() {
			this.hist.clear();
		}
	}

}
