package w5.ex1.iterator.sol;

import java.util.Date;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Random;

public class DummyDates implements Iterable<Date> {

	private final static int SIZE = 15;
	private Date[] dates = new Date[SIZE];
	private int actualSize;

	public DummyDates() {
		Random r = new Random();
		// Fill the array....
		for (int i = 0; i < SIZE; i++) {
			dates[i] = new Date(r.nextInt());
		}
		actualSize = SIZE;
	}

	public Iterator<Date> iterator() {
		return new DSIterator();
	}

	private class DSIterator implements Iterator<Date> {

		// Start stepping through the array from the beginning
		private int next = 0;
		private boolean nextCalled = false;

		public boolean hasNext() {
			// Check if a current element is the last in the array
			return (next < actualSize);
		}

		/**
		 * @pre hasNext() returns true
		 */
		public Date next() {
			if (hasNext()) {
				nextCalled = true;
				return dates[next++];
			} else {
				throw new NoSuchElementException();
			}
		}

		/**
		 * @pre next must have been called after last call to remove
		 */
		public void remove() {
			if (!nextCalled) {
				throw new IllegalStateException();
			}
			for (int i = next - 1; i < actualSize - 1; i++) {
				dates[i] = dates[i + 1];
			}
			dates[SIZE - 1] = null; // Prevent potential memory leak.
			actualSize--;
			next--;
			nextCalled = false;
		}

	}
}
